﻿using Microsoft.Azure.Management.Compute.Fluent;
using Microsoft.Azure.Management.Fluent;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.IO;
using System.Linq;
using System.Net.Http;
using System.Text;
using System.Threading.Tasks;

namespace ConsoleApp1
{
    class Program
    {
        static void Main(string[] args)
        {
            string credFile = @"D:\Settings\azure_newuser1_prop.txt";

            UsingRESTFulAPI(credFile);
            UsingManagedLib(credFile);

            Console.ReadLine();
        }

        private static void UsingManagedLib(string credFile)
        {
            IAzure azure = Azure.Authenticate(credFile).WithDefaultSubscription();

            var resGroups = azure.ResourceGroups.List();
            Console.WriteLine(resGroups.Count());

            var sysnetres = azure.ResourceGroups.GetByName("sysnet_res");

            foreach (var resGroup in resGroups)
            {
                Console.WriteLine(resGroup.Name);

                var vmList = azure.VirtualMachines.ListByResourceGroup(resGroup.Name);
                Console.WriteLine($"\t[VirtualMachines: {vmList.Count()}]");
                foreach (var vm in vmList)
                {
                    Console.WriteLine("\t\t" + vm.Name);
                }

                var deployments = azure.Deployments.ListByResourceGroup(resGroup.Name);
                Console.WriteLine($"\t[Deployments: {deployments.Count()}]");
                foreach (var deployment in deployments)
                {
                    Console.WriteLine("\t\t" + deployment.Name);
                }

                var containers = azure.ContainerGroups.ListByResourceGroup(resGroup.Name);
                Console.WriteLine($"\t[ContainerGroups: {containers.Count()}]");
                foreach (var container in containers)
                {
                    Console.WriteLine("\t\t" + container.Name);
                }

                var vmScaleSets = azure.VirtualMachineScaleSets.ListByResourceGroup(resGroup.Name);
                Console.WriteLine($"\t[VMScaleSets: {vmScaleSets.Count()}]");
                foreach (var vmScaleSet in vmScaleSets)
                {
                    Console.WriteLine("\t\t" + vmScaleSet.Name);
                }

                var webApps = azure.WebApps.ListByResourceGroup(resGroup.Name);
                Console.WriteLine($"\t[WebApps: {webApps.Count()}]");
                foreach (var webApp in webApps)
                {
                    Console.WriteLine("\t\t" + webApp.Name);
                }

                var containerServices = azure.ContainerServices.ListByResourceGroup(resGroup.Name);
                Console.WriteLine($"\t[ContainerServices: {containerServices.Count()}]");
                foreach (var containerService in containerServices)
                {
                    Console.WriteLine("\t\t" + containerService.Name);
                }

                var availabilitySets = azure.AvailabilitySets.ListByResourceGroup(resGroup.Name);
                Console.WriteLine($"\t[AvailabilitySets: {availabilitySets.Count()}]");
                foreach (var availabilitySet in availabilitySets)
                {
                    Console.WriteLine("\t\t" + availabilitySet.Name);
                }

                var disks = azure.Disks.ListByResourceGroup(resGroup.Name);
                Console.WriteLine($"\t[Disks: {disks.Count()}]");
                foreach (var disk in disks)
                {
                    Console.WriteLine("\t\t" + disk.Name);
                }

                var loadBalancers = azure.LoadBalancers.ListByResourceGroup(resGroup.Name);
                Console.WriteLine($"\t[LoadBalancers: {loadBalancers.Count()}]");
                foreach (var loadBalancer in loadBalancers)
                {
                    Console.WriteLine("\t\t" + loadBalancer.Name);
                }

                var managementLocks = azure.ManagementLocks.ListByResourceGroup(resGroup.Name);
                Console.WriteLine($"\t[ManagementLocks: {managementLocks.Count()}]");
                foreach (var managementLock in managementLocks)
                {
                    Console.WriteLine("\t\t" + managementLock.ToString());
                }

                var publicIPAddresses = azure.PublicIPAddresses.ListByResourceGroup(resGroup.Name);
                Console.WriteLine($"\t[PublicIPAddresses: {publicIPAddresses.Count()}]");
                foreach (var publicIPAddress in publicIPAddresses)
                {
                    Console.WriteLine("\t\t" + publicIPAddress.ToString());
                }

                var searchServices = azure.SearchServices.ListByResourceGroup(resGroup.Name);
                Console.WriteLine($"\t[SearchServices: {searchServices.Count()}]");
                foreach (var searchService in searchServices)
                {
                    Console.WriteLine("\t\t" + searchService.ToString());
                }

                var snapshots = azure.Snapshots.ListByResourceGroup(resGroup.Name);
                Console.WriteLine($"\t[Snapshots: {snapshots.Count()}]");
                foreach (var snapshot in snapshots)
                {
                    Console.WriteLine("\t\t" + snapshot.ToString());
                }

                var storageAccounts = azure.StorageAccounts.ListByResourceGroup(resGroup.Name);
                Console.WriteLine($"\t[StorageAccounts: {storageAccounts.Count()}]");
                foreach (var storageAccount in storageAccounts)
                {
                    Console.WriteLine("\t\t" + storageAccount.ToString());
                }

                var virtualMachineCustomImages = azure.VirtualMachineCustomImages.ListByResourceGroup(resGroup.Name);
                Console.WriteLine($"\t[VirtualMachineCustomImages: {virtualMachineCustomImages.Count()}]");
                foreach (var virtualMachineCustomImage in virtualMachineCustomImages)
                {
                    Console.WriteLine("\t\t" + virtualMachineCustomImage.Name);
                }
            }

            // Azure에서 제공되는 템플릿 목록
            //var computeSkus = azure.ComputeSkus.ListByResourceType(ComputeResourceType.VirtualMachines);
            //Console.WriteLine($"\t[ComputeSkus: {computeSkus.Count()}]");
            //foreach (var computeSku in computeSkus)
            //{
            //    Console.WriteLine("\t\t" + computeSku.Name);
            //}
        }

        private static async void UsingRESTFulAPI(string credFile)
        {
            AzureCredential credential = GetCredentialFromFile(credFile);

            string accessToken = await GetAccessToken(credential);
            Console.WriteLine("AccessToken: " + accessToken);

            string resourceList = await GetResourceGroups(credential, accessToken);
            Console.WriteLine(resourceList);
        }

        private static AzureCredential GetCredentialFromFile(string filePath)
        {
            KeyValueReader reader = new KeyValueReader(filePath);
            AzureCredential credential = new AzureCredential();

            credential.subscriptionId = reader["subscription"];
            credential.clientId = reader["client"];
            credential.clientSecret = reader["key"];
            credential.tenantId = reader["tenant"];

            credential.managementEndpointUrl = reader["managementURI"];
            credential.activeDirectoryEndpointUrl = "https://login.microsoftonline.com";
            credential.activeDirectoryGraphResourceId = reader["graphURL"];
            credential.resourceManagerEndpointUrl = reader["baseURL"];
            credential.sqlManagementEndpointUrl = "https://management.core.windows.net:8443/";
            credential.galleryEndpointUrl = "https://gallery.azure.com/";

            return credential;
        }

        private static async Task<string> GetResourceGroups(AzureCredential credential, string accessToken)
        {
            HttpClient hc = new HttpClient();
            hc.DefaultRequestHeaders.Add("Authorization", "Bearer " + accessToken);

            string url = $"https://management.azure.com/subscriptions/{credential.subscriptionId}/resourcegroups?api-version=2017-05-10";

            HttpResponseMessage hrm = await hc.GetAsync(url);
            string result = await hrm.Content.ReadAsStringAsync();

            return result;
        }

        private static async Task<string> GetAccessToken(AzureCredential credential)
        {
            HttpClient hc = new HttpClient();
            string url = "https://login.microsoftonline.com/" + credential.tenantId + "/oauth2/token";

            Dictionary<string, string> dict = new Dictionary<string, string>();
            dict.Add("resource", credential.resourceManagerEndpointUrl);
            dict.Add("client_id", credential.clientId);
            dict.Add("client_secret", credential.clientSecret);
            dict.Add("grant_type", "client_credentials");

            FormUrlEncodedContent content = new FormUrlEncodedContent(dict);
            HttpResponseMessage hrm = null;

            try
            {
                hrm = await hc.PostAsync(url, content);
            }
            catch (Exception e)
            {
                Console.WriteLine(e.ToString());
            }

            string result = await hrm.Content.ReadAsStringAsync();

            AzureAccessToken token = JsonConvert.DeserializeObject<AzureAccessToken>(result);

            return token.access_token;
        }
    }
}
